import 'package:logging/logging.dart';
import 'package:rook_users/rook_users.dart';
import 'package:rook_users/src/authorization/authorization_manager.dart';
import 'package:rook_users/src/authorization/get_basic_auth_from_credentials.dart';
import 'package:rook_users/src/data/preferences/users_preferences.dart';
import 'package:rook_users/src/data/server/model/create_rook_user_sv_model.dart';
import 'package:rook_users/src/data/server/model/delete_user_from_rook_sv_model.dart';
import 'package:rook_users/src/data/server/users_api_service.dart';
import 'package:rook_users/src/datasource/data_source.dart';
import 'package:rook_users/src/datasource/status/data_source_status.dart';
import 'package:rook_users/src/domain/environment/get_api_url_from_environment.dart';

/// Helper class to manage all users operations.
class RookUsersManager {
  final Logger _logger = Logger('RookUsersManager');

  late DataSourceStatus _dataSourceStatus;

  final UsersPreferences _usersPreferences = UsersPreferences();
  late UsersApiService _usersApiService;

  /// Create a new [RookUsersManager] instance.
  ///
  /// [apiUrl] - Users API url without https.
  RookUsersManager(String clientUUID, String secretKey) {
    final String authorization =
        getBasicAuthFromCredentials(clientUUID, secretKey);

    _dataSourceStatus = DataSourceStatus(
      getApiUrlFromEnvironment(
        RookUsersConfiguration.getEnvironment(),
        forceDevelop: false,
      ),
      authorization,
    );

    _usersApiService = UsersApiService(
      baseUrl: getApiUrlFromEnvironment(
        RookUsersConfiguration.getEnvironment(),
        forceDevelop: false,
      ),
      authorization: authorization,
    );
  }

  /// Retrieves the current user stored in preferences for the provided [UserType] or null if not found.
  Future<RookUser?> getUser(UserType userType) {
    return _usersPreferences.getRookUser(userType);
  }

  /// Registers provided userID in server and saves it to preferences.
  ///
  /// If the user was previously registered it will return the stored preferences value.
  ///
  /// * [userID] is defined by you, so feel free to use the same ID your app uses to identify it's users.
  /// * [userType] - Were the user account will be registered.
  Future<RookUser> registerRookUser(
    String userID,
    UserType userType,
  ) async {
    AuthorizationManager().checkAuthorization();

    final user = await _usersPreferences.getRookUser(userType);

    if (user != null && user.id == userID) {
      _logger.info('There is a user with the same ID, returning local user...');
      return user;
    }

    final newUser = RookUser(id: userID, type: userType);
    final createRookUserSvModel = CreateRookUserSvModel(
      sourceOfData: userType.identifier,
      userID: userID,
    );

    await _usersApiService.createRookUser(createRookUserSvModel);

    _logger.info('User: $newUser was registered successfully');

    await _usersPreferences.saveRookUser(newUser);

    _logger.info('User: $newUser was saved successfully');

    _dataSourceStatus.changeStatus(
        userID, getDataSourceFromUserType(userType), true);

    return newUser;
  }

  /// Removes user from the specified data source ([userType]),
  /// once removed rook servers won't accept any health data from the specified data source.
  Future<void> deleteUserFromRook(String userID, UserType userType) async {
    AuthorizationManager().checkAuthorization();

    final userToDelete = RookUser(id: userID, type: userType);
    final deleteUserFromRookSvModel = DeleteUserFromRookSvModel(
      dataSource: userToDelete.type.identifier,
    );

    await _usersApiService.deleteUserFromRook(
        userID, deleteUserFromRookSvModel);

    _logger.info('User: $userToDelete was deleted from rook servers');

    final success = await _usersPreferences.deleteRookUser(userToDelete.type);

    _logger.info('User: $userToDelete was deleted from preferences? $success');
  }
}
